@@ -23,6 +23,7 @@ import re
 import sys
 import textwrap
 
+from pyomo.version.info import releaselevel
 from pyomo.common.deprecation import deprecated
 from pyomo.common.fileutils import PYOMO_ROOT_DIR
 
@@ -30,6 +31,18 @@ _indentation_re = re.compile(r'\s*')
 _bullet_re = re.compile(r'(?:[-*] +)|(\[\s*[A-Za-z0-9\.]+\s*\] +)')
 _bullet_char = '-*['
 
+_RTD_URL = "https://pyomo.readthedocs.io/en/%s/errors.html" % (
+    'stable'
+    if (releaselevel == 'final'
+        or 'sphinx' in sys.modules
+        or 'Sphinx' in sys.modules)
+    else 'latest')
+
+def RTD(_id):
+    _id = str(_id).lower()
+    assert _id[0] in 'wex'
+    return f"{_RTD_URL}#{_id}"
+
 _DEBUG = logging.DEBUG
 _NOTSET = logging.NOTSET
 if not __debug__:
@@ -87,10 +100,14 @@ class WrappingFormatter(logging.Formatter):
         super(WrappingFormatter, self).__init__(**kwds)
 
     def format(self, record):
-        _orig = {k:getattr(record, k) for k in ('msg', 'args', 'pathname')}
+        _orig = {k: getattr(record, k)
+                 for k in ('msg', 'args', 'pathname', 'levelname')}
+        _id = getattr(record, 'id', None)
         msg = record.getMessage()
         record.msg = self._flag
         record.args = None
+        if _id:
+            record.levelname += f" ({_id.upper()})"
         if self.basepath and record.pathname.startswith(self.basepath):
             record.pathname = '[base]' + record.pathname[len(self.basepath):]
         try:
@@ -125,13 +142,16 @@ class WrappingFormatter(logging.Formatter):
         # recombine, substituting and wrapping any lines that contain
         # _flag.
         return '\n'.join(
-            self._wrap_msg(l, msg) if self._flag in l else l
+            self._wrap_msg(l, msg, _id) if self._flag in l else l
             for l in raw_msg.splitlines()
         )
 
-    def _wrap_msg(self, l, msg):
+    def _wrap_msg(self, l, msg, _id):
         indent = _indentation_re.match(l).group()
-        return self._wrap(l.strip().replace(self._flag, msg), indent)
+        wrapped_msg = self._wrap(l.strip().replace(self._flag, msg), indent)
+        if _id:
+            wrapped_msg += f"\n{indent}    See also {RTD(_id)}"
+        return wrapped_msg
 
     def _wrap(self, msg, base_indent):
         # As textwrap only works on single paragraphs, we need to break
