@@ -15,8 +15,8 @@ def main():
 
     # solve
     solver = MumpsCentralizedAssembledLinearSolver()
-    x1 = solver.solve(A, b1)
-    x2 = solver.solve(A, b2)
+    x1, res = solver.solve(A, b1)
+    x2, res = solver.solve(A, b2)
     assert np.allclose(x1, true_x1)
     assert np.allclose(x2, true_x2)
 
@@ -24,8 +24,8 @@ def main():
     solver = MumpsCentralizedAssembledLinearSolver()
     solver.do_symbolic_factorization(A)
     solver.do_numeric_factorization(A)
-    x1 = solver.do_back_solve(b1)
-    x2 = solver.do_back_solve(b2)
+    x1, res = solver.do_back_solve(b1)
+    x2, res = solver.do_back_solve(b2)
     assert np.allclose(x1, true_x1)
     assert np.allclose(x2, true_x2)
 
@@ -34,20 +34,20 @@ def main():
     # and upper portions of the matrix are given.
     solver = MumpsCentralizedAssembledLinearSolver(sym=2)
     A_lower_triangular = sp.tril(A)
-    x1 = solver.solve(A_lower_triangular, b1)
+    x1, res = solver.solve(A_lower_triangular, b1)
     assert np.allclose(x1, true_x1)
 
     # Tell Mumps the matrix is symmetric and positive-definite
     solver = MumpsCentralizedAssembledLinearSolver(sym=1)
     A_lower_triangular = sp.tril(A)
-    x1 = solver.solve(A_lower_triangular, b1)
+    x1, res = solver.solve(A_lower_triangular, b1)
     assert np.allclose(x1, true_x1)
 
     # Set options
     solver = MumpsCentralizedAssembledLinearSolver(icntl_options={11: 2}) # compute error stats
     solver.set_cntl(2, 1e-4) # set the stopping criteria for iterative refinement
     solver.set_icntl(10, 5) # set the maximum number of iterations for iterative refinement to 5
-    solver.solve(A, b1)
+    x1, res = solver.solve(A, b1)
     assert np.allclose(x1, true_x1)
 
     # Get information after the solve
