@@ -13,6 +13,7 @@ __all__ = ('Suffix',
            'active_import_suffix_generator')
 
 import logging
+from typing import overload
 
 from pyomo.common.collections import ComponentMap
 from pyomo.common.log import is_debug_set
@@ -168,6 +169,10 @@ class Suffix(ComponentMap, ActiveComponent):
                            INT: 'Suffix.INT',
                            None: str(None)}
 
+    @overload
+    def __init__(self, *, direction=LOCAL, datatype=FLOAT,
+                 initialize=None, rule=None, name=None, doc=None): ...
+
     def __init__(self, **kwds):
 
         # Suffix type information
