@@ -23,21 +23,15 @@ from pyomo.core.base import (SortComponents,
 from pyomo.core.base.numvalue import (is_fixed,
                                       value,
                                       ZeroConstant)
-from pyomo.core.base.plugin import ModelComponentFactory
+from pyomo.core.base.component import ModelComponentFactory
 from pyomo.core.base.constraint import (Constraint,
                                         IndexedConstraint,
-                                        SimpleConstraint,
+                                        ScalarConstraint,
                                         _ConstraintData)
 from pyomo.core.expr.numvalue import native_numeric_types
 from pyomo.repn import generate_standard_repn
 
-from six import iteritems, PY3
-from six.moves import xrange
-
-if PY3:
-    from collections.abc import Mapping as collections_Mapping
-else:
-    from collections import Mapping as collections_Mapping
+from collections.abc import Mapping
 
 
 logger = logging.getLogger('pyomo.core')
@@ -159,12 +153,12 @@ def compile_block_linear_constraints(parent_block,
 
             else:
 
-                singleton = isinstance(constraint, SimpleConstraint)
+                singleton = isinstance(constraint, ScalarConstraint)
 
                 # Note that as we may be removing items from the _data
                 # dictionary, we need to make a copy of the items list
                 # before iterating:
-                for index, constraint_data in list(iteritems(constraint)):
+                for index, constraint_data in list(constraint.items()):
 
                     if constraint_data.body.__class__ in native_numeric_types or constraint_data.body.polynomial_degree() <= 1:
 
@@ -459,7 +453,7 @@ class _LinearMatrixConstraintData(_LinearConstraintData):
         vals = comp._vals
         try:
             return sum(varmap[jcols[p]]() * vals[p]
-                       for p in xrange(prows[index],
+                       for p in range(prows[index],
                                        prows[index+1]))
         except (ValueError, TypeError):
             if exception:
@@ -517,7 +511,7 @@ class _LinearMatrixConstraintData(_LinearConstraintData):
         if prows[self._index] == prows[self._index+1]:
             return()
         variables = tuple(varmap[jcols[p]]
-                          for p in xrange(prows[self._index],
+                          for p in range(prows[self._index],
                                           prows[self._index+1])
                           if not varmap[jcols[p]].fixed)
 
@@ -533,7 +527,7 @@ class _LinearMatrixConstraintData(_LinearConstraintData):
         varmap = comp._varmap
         if prows[self._index] == prows[self._index+1]:
             return ()
-        coefs = tuple(vals[p] for p in xrange(prows[self._index],
+        coefs = tuple(vals[p] for p in range(prows[self._index],
                                               prows[self._index+1])
                       if not varmap[jcols[p]].fixed)
 
@@ -553,7 +547,7 @@ class _LinearMatrixConstraintData(_LinearConstraintData):
         if prows[self._index] == prows[self._index+1]:
             return 0
         terms = tuple(vals[p] * varmap[jcols[p]]()
-                      for p in xrange(prows[self._index],
+                      for p in range(prows[self._index],
                                       prows[self._index+1])
                       if varmap[jcols[p]].fixed)
 
@@ -575,7 +569,7 @@ class _LinearMatrixConstraintData(_LinearConstraintData):
         if prows[self._index] == prows[self._index+1]:
             return ZeroConstant
         return sum(varmap[jcols[p]] * vals[p]
-                   for p in xrange(prows[index],
+                   for p in range(prows[index],
                                    prows[index+1]))
 
     @property
@@ -623,8 +617,7 @@ class _LinearMatrixConstraintData(_LinearConstraintData):
 
 @ModelComponentFactory.register(
                    "A set of constraint expressions in Ax=b form.")
-class MatrixConstraint(collections_Mapping,
-                       IndexedConstraint):
+class MatrixConstraint(Mapping, IndexedConstraint):
 
     #
     # Bound types
@@ -679,7 +672,7 @@ class MatrixConstraint(collections_Mapping,
 
         _init = _LinearMatrixConstraintData
         self._data = tuple(_init(i, component=self)
-                           for i in xrange(len(self._range_types)))
+                           for i in range(len(self._range_types)))
 
     #
     # Override some IndexedComponent methods
@@ -692,7 +685,7 @@ class MatrixConstraint(collections_Mapping,
         return self._data.__len__()
 
     def __iter__(self):
-        return iter(i for i in xrange(len(self)))
+        return iter(i for i in range(len(self)))
 
     #
     # Remove methods that allow modifying this constraint
